import { FSWatcher } from 'chokidar';
import { Context } from 'koa';
import { IWatchAppInfo } from './';
interface IBuildParams {
    /**
     * 低码应用 ID
     */
    appId: string;
    /**
     * 构建产物类型 mp/web
     */
    buildTypeList: string[];
    /**
     * 小程序构建类型
     */
    generateMpType?: 'app' | 'subpackage';
    /**
     * 小程序部署参数
     */
    deployOptions: {
        mode: 'preview' | 'upload';
        version: string;
        description: string;
        appHistoryId?: number;
    };
    /**
     * 插件
     */
    plugins: any[];
    /**
     * 低码应用模板
     */
    mainAppSerializeData: string;
    /**
     * 低码子应用模板
     */
    subAppSerializeDataStrList: any[];
    /**
     * 依赖, 如 kone
     */
    dependencies: any[];
    /**
     * 低码应用 环境ID
     */
    envId: string;
    /**
     * 数据源
     */
    datasources: any;
}
declare type DistType = 'web' | 'qrcode' | 'dist';
interface ICheckStatusParams {
    buildId: string;
    distType: DistType[];
    appId: string;
}
interface ICheckStatusDist {
    web?: string;
    dist?: string;
    qrcode?: string;
}
interface IDeployHistoryParams {
    appId: string;
    pageSize?: number;
    pageIndex?: number;
}
interface IDeployDetailParams {
    appId: string;
    idx: string;
}
declare enum BUILD_STATUE {
    BUILDING = "building",
    BUILD_FAILED = "buildFail",
    BUILD_SUCCESS = "success",
    INSTALL_FAILED = "fail"
}
export declare class Builder {
    protected _workspace: string;
    protected _info: IWatchAppInfo;
    constructor(workspace: string, info: IWatchAppInfo);
    build(ctx: Context, params: IBuildParams): Promise<{
        buildId: string;
    }>;
    _writeFiles(appId: string, buildId: string, contents: any[]): Promise<void>;
    checkStatusInLocal(ctx: Context, params: ICheckStatusParams): Promise<{
        web?: string;
        dist?: string;
        qrcode?: string;
        status: any;
    }>;
    _getAllDist(params: ICheckStatusParams, inputs: IBuildParams): Promise<ICheckStatusDist>;
    deployHistory(ctx: Context, params: IDeployHistoryParams): Promise<{
        count: number;
        rows: {
            appId: string;
            idx: string;
            ciId: string;
            createdAt: string;
            status: any;
            preview: Partial<{
                web?: string;
                dist?: string;
                qrcode?: string;
                status: any;
            } | {
                status: BUILD_STATUE;
            }>;
        }[];
    }>;
    deployDetail(ctx: Context, params: IDeployDetailParams): Promise<{
        appId: string;
        idx: string;
        ciId: string;
        createdAt: string;
        status: any;
        preview: Pick<{
            web?: string;
            dist?: string;
            qrcode?: string;
            status: any;
        }, "web" | "qrcode" | "dist">;
    }>;
}
export declare class Watcher {
    protected _workspace: string;
    protected _watcher?: FSWatcher;
    constructor(workspace: string);
    start(): void;
    _callFramework(projectPath: string, command?: string): Promise<unknown>;
    /**
     * 设置构建状态
     */
    _setStatus(path: string, status: BUILD_STATUE): void;
    /**
     * 清除源码
     */
    _clean(path: string): void;
    end(): void;
}
export {};
//# sourceMappingURL=service.d.ts.map