"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUsedComps = void 0;
const cals_1 = require("@cloudbase/cals");
// 入口
function getUsedComps(
// 主子包的应用内容
apps, 
// 分库后的组件库
comps) {
    // 0. 应用内容转换(cals to weapps)
    const transferApps = apps.map((app) => {
        return (0, cals_1.deserializePlatformApp)(app, { dependencies: comps });
    });
    // 1. 寻找应用内容里用到的所有组件
    // 1.1 寻找主子应用各自的内容里所用到的组件
    const appUsedComps = transferApps.map((app) => {
        var _a;
        const usedComps = {};
        // 主子包模式下, 可能没有应用内容
        (_a = app.pageInstanceList) === null || _a === void 0 ? void 0 : _a.forEach((p) => {
            getUsedCompsFromProperties(p.componentInstances, usedComps);
        });
        return usedComps;
    });
    // 1.2 合并主子包所有用到的组件
    const mergeAppUsedComps = appUsedComps.reduce((comps, item) => {
        for (const libName of Object.keys(item)) {
            comps[libName] = new Set([...Array.from(item[libName]), ...Array.from(comps[libName] || [])]);
        }
        return comps;
    }, {});
    // 2. 寻找应用内容的复合组件里嵌套的其他组件(复合组件即低码组件)
    // 2.1 过滤出复合组件库
    const compositedLibMap = new Map();
    comps.forEach((lib) => {
        if (lib.isComposite) {
            compositedLibMap.set(lib.name, lib);
        }
    });
    // 2.2 递归查询复合组件库所使用的组件
    const libUsedComps = {};
    getUsedCompsFromCompositedLibsRecursively(mergeAppUsedComps, [], compositedLibMap, libUsedComps);
    // 3. 合并[应用内容用到的组件]和[复合组件库用到的组件]
    const allUsedComps = [mergeAppUsedComps, libUsedComps].reduce((comps, item) => {
        for (const libName of Object.keys(item)) {
            comps[libName] = new Set([...Array.from(item[libName]), ...Array.from(comps[libName] || [])]);
        }
        return comps;
    }, {});
    return allUsedComps;
}
exports.getUsedComps = getUsedComps;
// 递归查询属性树所使用的组件
function getUsedCompsFromProperties(properties, usedComps) {
    for (const id of Object.keys(properties)) {
        if (Object.prototype.hasOwnProperty.call(properties, id)) {
            const { xComponent, properties: children } = properties[id];
            if (xComponent) {
                // 记录使用到的组件库和组件
                const { moduleName: libName, name: compName } = xComponent;
                if (!usedComps[libName]) {
                    usedComps[libName] = new Set();
                }
                usedComps[libName].add(compName);
            }
            if (children) {
                // 遍历下一层属性树
                getUsedCompsFromProperties(children, usedComps);
            }
        }
    }
}
// 递归查询复合组件库所使用的组件
function getUsedCompsFromCompositedLibsRecursively(
// 已知的被使用到的组件
usedComps, 
// 已知的被遍历过的组件
checkedComps, 
// 全量复合组件库
compositedLibMap, 
// 新增的被使用到的组件
addedUsedComps) {
    // 开始遍历已知的被使用到的组件库
    Object.keys(usedComps).forEach((libName) => {
        if (!compositedLibMap.has(libName)) {
            // 不是复合组件库, 不需要继续遍历
            return;
        }
        // 遍历复合组件库
        const lib = compositedLibMap.get(libName);
        const cmpNames = usedComps[libName];
        cmpNames.forEach((cmpName) => {
            // 找到复合组件的内容
            const cmp = lib.components.find((c) => c.name === cmpName);
            if (!cmp) {
                // 找不到复合组件内容, 应该抛异常
                console.warn('Component not found', `${libName}:${cmpName}`);
                return;
            }
            if (checkedComps.indexOf(cmp) > -1) {
                // 该复合组件已经在其他递归路径上入栈
                return;
            }
            // 该复合组件第一次入栈, 则需要记录并继续递归它的内容
            checkedComps.push(cmp);
            const usedCompsOfThisComp = {};
            // 和遍历应用内容一样, 遍历低码组件内容, 寻找出组件自身引用的所有组件
            getUsedCompsFromProperties(cmp.componentInstances, usedCompsOfThisComp);
            Object.keys(usedCompsOfThisComp).forEach((libName) => {
                // 开始记录新增的被使用的组件库
                if (!addedUsedComps[libName]) {
                    addedUsedComps[libName] = usedCompsOfThisComp[libName];
                }
                else {
                    usedCompsOfThisComp[libName].forEach((n) => addedUsedComps[libName].add(n));
                }
            });
            // 递归遍历
            getUsedCompsFromCompositedLibsRecursively(usedCompsOfThisComp, checkedComps, compositedLibMap, addedUsedComps);
        });
    });
}
