"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.KoaWSServer = exports.WEDA_WORKSPACES_PATH = exports.KoaServer = void 0;
const koa_1 = __importDefault(require("koa"));
const koa_bodyparser_1 = __importDefault(require("koa-bodyparser"));
const router_1 = __importDefault(require("@koa/router"));
const cors_1 = __importDefault(require("@koa/cors"));
const koa_websocket_1 = __importDefault(require("koa-websocket"));
const toolbox_1 = require("@cloudbase/toolbox");
const koa_mount_1 = __importDefault(require("koa-mount"));
const koa_static_1 = __importDefault(require("koa-static"));
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const chokidar_1 = __importDefault(require("chokidar"));
const dayjs_1 = __importDefault(require("dayjs"));
const os_1 = __importDefault(require("os"));
const cals_1 = require("@cloudbase/cals");
const chalk_1 = __importDefault(require("chalk"));
const lodash_1 = require("lodash");
const debug_1 = __importDefault(require("debug"));
const Diff3 = __importStar(require("node-diff3"));
const utils_1 = require("./utils");
const WS = __importStar(require("ws"));
const debug = (0, debug_1.default)('server');
class KoaServer {
    constructor(info) {
        this.apiApp = new koa_1.default();
        this.app = new koa_1.default();
        this.router = new router_1.default();
        this.staticApp = new koa_1.default();
        this.info = info;
    }
    addListener(path, callback) {
        this.router.post(path, async (ctx, next) => {
            await callback(ctx);
            await next();
        });
    }
    listen() {
        this.initApp();
        return this.app.listen(this.info.port);
    }
    initApiApp() {
        // 回包方法封装
        this.apiApp.use(async (ctx, next) => {
            ctx.success = (data = {}) => {
                ctx.response.body = {
                    data,
                    errcode: 0,
                };
            };
            ctx.fail = (errmsg = '', errcode = 1) => {
                ctx.response.body = {
                    errmsg,
                    errcode,
                };
            };
            ctx.set('Access-Control-Allow-Private-Network', 'true');
            await next();
        });
        // 异常捕获
        this.apiApp.use(async (ctx, next) => {
            try {
                await next();
            }
            catch (e) {
                if (e instanceof toolbox_1.CloudBaseError) {
                    ctx.fail(e.message);
                }
                else {
                    throw e;
                }
            }
        });
        // 跨域
        const cors = (0, cors_1.default)({
            origin: '*',
        });
        this.apiApp.use(cors);
        // 解析器
        const parser = (0, koa_bodyparser_1.default)({
            jsonLimit: '5mb',
        });
        this.apiApp.use(parser);
        // api
        this.apiApp.use(this.router.routes());
    }
    initStaticApp() {
        if (this.info.staticPath) {
            // 挂载Static路径
            this.staticApp.use((0, koa_static_1.default)(this.info.staticPath));
        }
    }
    initApp() {
        // api
        this.initApiApp();
        this.app.use((0, koa_mount_1.default)('/v1', this.apiApp));
        this.app.use((0, koa_mount_1.default)('/api/v1', this.apiApp));
        // static
        this.initStaticApp();
        this.app.use((0, koa_mount_1.default)('/assets', this.staticApp));
        this.app.use((0, koa_mount_1.default)('/static', this.staticApp));
    }
}
exports.KoaServer = KoaServer;
function debounceFnFactory(fn, wait, ...args) {
    const id = `${fn.name}:${JSON.stringify(args)}`;
    debounceFnFactory[id] = debounceFnFactory[id] || (0, lodash_1.debounce)(fn.bind(this, ...args), wait);
    return debounceFnFactory[id];
}
exports.WEDA_WORKSPACES_PATH = path_1.default.resolve(os_1.default.homedir(), '.weda-workspaces');
class KoaWSServer {
    constructor() {
        this.app = (0, koa_websocket_1.default)(new koa_1.default());
        this.wedaAppClients = {};
        this.Logger = {
            warn(appId, clientId, info, noPrefix = false) {
                this.log(appId, clientId, info, noPrefix, 'yellow');
            },
            info(appId, clientId, info, noPrefix = false) {
                this.log(appId, clientId, info, noPrefix, 'white');
            },
            error(appId, clientId, info, noPrefix = false) {
                this.log(appId, clientId, info, noPrefix, 'red');
            },
            success(appId, clientId, info, noPrefix = false) {
                this.log(appId, clientId, info, noPrefix, 'green');
            },
            log(appId, clientId, info, noPrefix = false, color) {
                console.log(chalk_1.default[color](`${noPrefix ? '' : `[${(0, dayjs_1.default)().format('HH:mm:ss')}][${appId}][${clientId}] `}${info}`));
            }
        };
        this.app.ws.use((ctx) => {
            ctx.websocket.on('message', async (message) => {
                try {
                    const objMsg = JSON.parse(message.toString());
                    const { appId, envId, clientId } = objMsg.data;
                    this.handleAppSession(appId, ctx.websocket, clientId);
                    switch (objMsg.event) {
                        // 收到编辑器的更新通知，更新本地代码
                        case 'cals-updated':
                            debug(`>>> receive cals-update event: ${appId} ${clientId}`);
                            this.wedaAppClients[appId][clientId].lastedUpdateCals = objMsg.data.calsData;
                            debounceFnFactory(this.handleCalsUpdated.bind(this), 1000, appId, envId, clientId)();
                            break;
                        // 合并到本地代码
                        case 'merge-into-local':
                            debug(`>>> receive merge-into-local event: ${appId} ${clientId}`);
                            const { browserCalsData, baseCalsData, originHistoryId } = objMsg.data;
                            let mergeCodeList = await this.findBenchmarkAndMergeCode(appId, baseCalsData, browserCalsData, originHistoryId, envId, clientId);
                            const isConflict = !!mergeCodeList.find((item) => item.conflict);
                            const wsMsg = {
                                event: 'merge-into-local-result',
                                data: {
                                    appId,
                                    envId,
                                    clientId,
                                    conflict: isConflict,
                                    // 如果 app-config 或 page-config.json 有冲突，codeToCals 就会执行异常
                                    calsData: isConflict ? {} : (0, cals_1.codeToCals)(mergeCodeList, 'v0'),
                                    codeList: mergeCodeList,
                                },
                            };
                            this.wedaAppClients[appId][clientId].mergeHappend = true;
                            ctx.websocket.send(JSON.stringify(wsMsg));
                            this.Logger.info(appId, clientId, '代码合并: WeDa 在线编辑器 => 本地');
                            // 发生合并后无论代码有无改变都执行一遍（零的逻辑），保持逻辑一致
                            setTimeout(() => {
                                debounceFnFactory(this.notifyCodeUpdated.bind(this), 1000, appId, envId, clientId)();
                            }, 300);
                            break;
                        // 自动解决冲突
                        case 'auto-resolve-conflict':
                            const { codeList, from } = objMsg.data;
                            await this.autoResolveConfilct(appId, codeList, from, envId, clientId);
                            break;
                        // 通过 originHistoryId 获取 cals 的结果
                        case 'fetch-cals-by-origin-id-result':
                            const { appId: _appId, calsData: _calsData } = objMsg.data;
                            this.wedaAppClients[_appId][clientId]['fetch-cals-by-origin-id-result'] = _calsData;
                            break;
                        // 写入应用 DTS 类型定义文件
                        case 'sync-dts-to-local':
                            const { dts } = objMsg.data;
                            await this.syncDTS(appId, envId, dts);
                            break;
                    }
                }
                catch (e) {
                    console.error(e);
                }
            });
        });
    }
    async autoResolveConfilct(appId, codeList, from, envId, clientId) {
        debug(`>>> call autoResolveConfilct: ${appId} ${clientId}`);
        // 未发生合并不进行同步
        if (!this.wedaAppClients[appId][clientId].mergeHappend) {
            debug('>>> call autoResolveConfilct: mergeHappend 为 false，不进行自动冲突解决');
            return;
        }
        const conflictCodelist = codeList.filter((item) => item.conflict);
        const projectPath = this.getProjectPath(appId, envId);
        await Promise.all(conflictCodelist.map((codeItem) => {
            const resultCode = (0, cals_1.autoResolveConflicts)(codeItem.code, from);
            return fs_extra_1.default.writeFile(path_1.default.resolve(projectPath, codeItem.path), resultCode, 'utf8');
        }));
    }
    handleAppSession(appId, websocket, clientId) {
        this.wedaAppClients[appId] = this.wedaAppClients[appId] || {};
        this.wedaAppClients[appId][clientId] = this.wedaAppClients[appId][clientId] || {
            watcher: null,
            changedFiles: [],
            websocket: websocket,
            lastedUpdateCals: null,
            mergeHappend: false,
            canSync: false, // 是否可进行同步操作（浏览器 与 本地）
        };
        // 同个 app 多个 ws 连接以最后一个为准
        if (this.wedaAppClients[appId][clientId].websocket !== websocket) {
            this.wedaAppClients[appId][clientId].websocket.close();
            this.wedaAppClients[appId][clientId].websocket = websocket;
        }
    }
    async findBenchmarkAndMergeCode(appId, baseCalsData, browserCalsData, browserOriginId, envId, clientId) {
        let localOriginHistoryId;
        let result = [];
        let conflictCodePaths = [];
        // 找基准
        const projectPath = this.getProjectPath(appId, envId);
        if (await fs_extra_1.default.pathExists(path_1.default.resolve(projectPath, 'src/app-config.json'))) {
            // 这里不能以 JSON 读，因为该文件可能存在冲突，导致 parse 异常
            const appConfigCode = await fs_extra_1.default.readFile(path_1.default.resolve(projectPath, 'src/app-config.json'), 'utf8');
            const matches = appConfigCode.match(/"originHistoryId": "(.*?)"/g);
            if (matches) {
                const ids = matches
                    .map((item) => +item.match(/\d+/))
                    .filter((item) => !!item)
                    .sort();
                // 取最小的
                localOriginHistoryId = ids[0] ? `${ids[0]}` : null;
            }
        }
        if (!localOriginHistoryId) {
            // local 没有 originHistoryId 的情况：以 browser 为准
            const browserCodeList = (0, cals_1.calsToCode)(browserCalsData, 'v0');
            result = browserCodeList;
        }
        else {
            let realBaseCalsData;
            // local 有 originHistoryId 的情况
            if (+localOriginHistoryId < +browserOriginId) {
                // local historyId < browser historyId
                realBaseCalsData = await this.fetchCalsDataByOriginHistoryId(appId, localOriginHistoryId, envId, clientId);
            }
            else {
                // local historyId >= browser historyId
                realBaseCalsData = baseCalsData;
            }
            const { mergeCodeList, conflictCodePaths: _conflictCodePaths } = await this.mergeCode(appId, realBaseCalsData, browserCalsData, envId);
            result = mergeCodeList;
            conflictCodePaths = _conflictCodePaths;
        }
        // 写入本地
        await this.writeToFiles(appId, result, envId);
        // 监听本地代码变化
        this.watchCodeChanged(appId, projectPath, result, envId, clientId);
        // 冲突提示
        if (conflictCodePaths.length > 0) {
            this.Logger.warn(appId, clientId, `冲突文件列表:\n${conflictCodePaths.map((item) => path_1.default.resolve(projectPath, item)).join('\n')}`);
        }
        return result;
    }
    fetchCalsDataByOriginHistoryId(appId, localOriginHistoryId, envId, clientId) {
        return new Promise((resolve, reject) => {
            const wsMsg = {
                event: 'fetch-cals-by-origin-id',
                data: {
                    appId,
                    envId,
                    clientId,
                    originHistoryId: localOriginHistoryId,
                },
            };
            this.wedaAppClients[appId][clientId].websocket.send(JSON.stringify(wsMsg));
            let retry = 8;
            const interval = setInterval(() => {
                retry--;
                const result = this.wedaAppClients[appId][clientId]['fetch-cals-by-origin-id-result'];
                if (result) {
                    clearInterval(interval);
                    this.wedaAppClients[appId][clientId]['fetch-cals-by-origin-id-result'] = null;
                    resolve(result);
                }
                else if (retry === 0) {
                    clearInterval(interval);
                    reject('fetchCalsDataByOriginHistoryId timeout');
                }
            }, 500);
        });
    }
    async mergeCode(appId, baseCalsData, browserCalsData, envId) {
        const projectPath = this.getProjectPath(appId, envId);
        // 将 baseCalsData 和 browserCalsData to codeList
        const baseCodeList = (0, cals_1.calsToCode)(baseCalsData, 'v0');
        const browserCodeList = (0, cals_1.calsToCode)(browserCalsData, 'v0');
        const localCodeList = await (0, utils_1.fileToCode)(projectPath);
        // 最全的代码文件列表
        const allFilePaths = (0, lodash_1.union)(browserCodeList.map((item) => item.path), baseCodeList.map((item) => item.path), localCodeList.map((item) => item.path));
        const mergeCodeList = [];
        const conflictCodePaths = [];
        for (let filePath of allFilePaths) {
            const foundBaseItem = baseCodeList.find((item) => item.path === filePath);
            const foundBrowserItem = browserCodeList.find((item) => item.path === filePath);
            const foundLoalItem = localCodeList.find((item) => item.path === filePath);
            if (foundBaseItem && foundBrowserItem && foundLoalItem) {
                // 判断本地文件是否已经有冲突，有冲突就不进行合并，否则嵌套的冲突会加大解决的难度
                const isConflict = await checkConflict(filePath);
                if (isConflict)
                    continue;
                // 内容更新 a o b
                const mergeResult = Diff3.merge(foundLoalItem.code, foundBaseItem.code, foundBrowserItem.code, {
                    stringSeparator: /[\r\n]+/,
                });
                mergeCodeList.push({
                    path: filePath,
                    code: mergeResult.result.join('\n'),
                    conflict: mergeResult.conflict,
                });
                if (mergeResult.conflict) {
                    conflictCodePaths.push(filePath);
                }
            }
            else {
                // 文件不一致
                if (foundBaseItem && foundBrowserItem && !foundLoalItem) {
                    // base 有; browser 有; local 没
                    // 内容一样 则删除文件
                    // 内容不一样，则为 browser 的内容
                    if (foundBaseItem.code !== foundBrowserItem.code) {
                        mergeCodeList.push(foundBrowserItem);
                    }
                }
                else if (foundBaseItem && !foundBrowserItem && !foundLoalItem) {
                    // base 有; browser 没; local 没
                    // 删除文件
                }
                else if (!foundBaseItem && foundBrowserItem && !foundLoalItem) {
                    // base 没; browser 有; local 没
                    // 文件为 browser 的文件
                    mergeCodeList.push(foundBrowserItem);
                }
                else if (!foundBaseItem && !foundBrowserItem && foundLoalItem) {
                    // base 没; browser 没; local 有
                    // 文件为 local 的文件
                    mergeCodeList.push(foundLoalItem);
                }
                else if (foundBaseItem && !foundBrowserItem && foundLoalItem) {
                    // base 有; browser 没; local 有
                    // 内容一样，则删除文件
                    // 内容不一样，则为 local 的内容
                    if (foundBaseItem.code !== foundLoalItem.code) {
                        mergeCodeList.push(foundLoalItem);
                    }
                }
                else if (!foundBaseItem && foundBrowserItem && foundLoalItem) {
                    // base 没; browser 有; local 有
                    // 文件存在，内容为 browser 和 local 的合并结果
                    // 判断本地文件是否已经有冲突，有冲突就不进行合并，否则嵌套的冲突会加大解决的难度
                    const isConflict = await checkConflict(filePath);
                    if (isConflict)
                        continue;
                    // FIXME: 这里的冲突结果是所有代码都冲突，无论代码有多相似，因为基准为空导致的。可能的优化就是使用 2-way 来合并
                    const mergeResult = Diff3.merge(foundLoalItem.code, '', foundBrowserItem.code, {
                        stringSeparator: /[\r\n]+/,
                    });
                    mergeCodeList.push({
                        path: filePath,
                        code: mergeResult.result.join('\n'),
                        conflict: mergeResult.conflict,
                    });
                    if (mergeResult.conflict) {
                        conflictCodePaths.push(filePath);
                    }
                }
            }
        }
        return { mergeCodeList, conflictCodePaths };
        async function checkConflict(filePath) {
            const localCodeContent = await fs_extra_1.default.readFile(path_1.default.resolve(projectPath, filePath), 'utf8');
            if ((0, cals_1.checkForConflicts)(localCodeContent)) {
                mergeCodeList.push({
                    path: filePath,
                    code: localCodeContent,
                    conflict: true,
                });
                conflictCodePaths.push(filePath);
                return true;
            }
            return false;
        }
    }
    async handleCalsUpdated(appId, envId, clientId) {
        debug(`>>> call handleCalsUpdated: ${appId} ${clientId}`);
        this.Logger.info(appId, clientId, '代码同步: WeDa 在线编辑器 >>> 本地');
        // 未发生合并不进行同步
        if (!this.wedaAppClients[appId][clientId].mergeHappend) {
            debug('>>> call handleCalsUpdated: mergeHappend 为 false，不进行本地代码写动作');
            return;
        }
        if (!this.wedaAppClients[appId][clientId].canSync) {
            debug('>>> call handleCalsUpdated: canSync 为 false，不进行本地代码写动作');
            return;
        }
        const calsJSON = this.wedaAppClients[appId][clientId].lastedUpdateCals;
        const codeList = (0, cals_1.calsToCode)(calsJSON, 'v0');
        // 写文件
        const projectPath = await this.writeToFiles(appId, codeList, envId);
        // 判断是否需要删除文件
        const localCodeList = await (0, utils_1.fileToCode)(this.getProjectPath(appId, envId), true);
        for (let localCodeItem of localCodeList) {
            const foundOne = codeList.find((item) => item.path === localCodeItem.path);
            if (!foundOne) {
                // 本地有而 codeList 没有，则需要删除掉
                await fs_extra_1.default.remove(path_1.default.resolve(this.getProjectPath(appId, envId), localCodeItem.path));
            }
        }
        // 监听本地代码
        this.watchCodeChanged(appId, projectPath, codeList, envId, clientId);
    }
    watchCodeChanged(appId, projectPath, codeList, envId, clientId) {
        if (!this.wedaAppClients[appId][clientId].watcher) {
            this.Logger.success(appId, clientId, `请使用你喜欢的 IDE 直接打开目录进行编辑\n`, true);
            this.Logger.success(appId, clientId, `应用本地代码目录: ${projectPath}\n`, true);
            this.wedaAppClients[appId][clientId].watcher = chokidar_1.default
                .watch(projectPath, {
                ignored: new RegExp(path_1.default.resolve(projectPath, '\\..*')),
                persistent: true,
            })
                .on('all', async (event, filePath) => {
                let needNotify = false;
                switch (event) {
                    case 'add':
                        const foundOne = codeList.find((codeItem) => path_1.default.resolve(projectPath, codeItem.path) === filePath);
                        // 初次监听会发出 add file 的 change 事件，应该排除掉
                        if (!foundOne) {
                            needNotify = true;
                        }
                        break;
                    case 'unlink': // delete
                    case 'change':
                        needNotify = true;
                        break;
                }
                if (needNotify) {
                    debug(`>>> Received file update messages that requires notification: ${appId} ${clientId}`);
                    this.wedaAppClients[appId][clientId].changedFiles.push({ event, filePath });
                    debounceFnFactory(this.notifyCodeUpdated.bind(this), 1000, appId, envId, clientId)();
                }
            });
        }
    }
    async writeToFiles(appId, codeList, envId) {
        debug('>>> call writeToFiles');
        const projectPath = this.getProjectPath(appId, envId);
        await Promise.all(codeList.map(async (codeItem) => {
            const codeFilePath = path_1.default.resolve(projectPath, codeItem.path);
            let codeContent = codeItem.code;
            await fs_extra_1.default.ensureFile(codeFilePath);
            if (path_1.default.extname(codeFilePath) === '.json') {
                // 格式化
                try {
                    codeContent = JSON.stringify(JSON.parse(codeContent), null, 2);
                }
                catch (e) {
                    // json 有冲突情况下 parse 异常
                    codeContent = codeContent;
                }
            }
            const originContent = await fs_extra_1.default.readFile(codeFilePath, 'utf8');
            if (originContent !== codeContent) {
                debug('>>> write file: ', codeFilePath);
                return fs_extra_1.default.writeFile(codeFilePath, codeContent, 'utf8');
            }
        }));
        return projectPath;
    }
    getProjectPath(appId, envId) {
        return path_1.default.resolve(exports.WEDA_WORKSPACES_PATH, envId, 'apps', appId);
    }
    async notifyCodeUpdated(appId, envId, clientId) {
        debug('>>> call notifyCodeUpdated');
        // 如果此时 websocket client 已经断开则不再发送
        if (this.wedaAppClients[appId][clientId].websocket._readyState !== WS.OPEN) {
            this.Logger.error(appId, clientId, `未检测到该应用的有效连接，已中止与该应用浏览器端代码的实时同步`);
            this.wedaAppClients[appId][clientId].watcher && await this.wedaAppClients[appId][clientId].watcher.close();
            this.wedaAppClients[appId][clientId].watcher = null;
            return;
        }
        // 未发生合并不进行同步
        if (!this.wedaAppClients[appId][clientId].mergeHappend) {
            debug('>>> call notifyCodeUpdated: mergeHappend 为 false，不发送 code-updated 事件');
            return;
        }
        const projectPath = this.getProjectPath(appId, envId);
        const codeList = await (0, utils_1.fileToCode)(projectPath);
        // 代码里存在未解决的冲突不进行同步
        const foundConflictItem = codeList.find((item) => (0, cals_1.checkForConflicts)(item.code));
        if (foundConflictItem) {
            this.Logger.warn(appId, clientId, `代码有冲突未解决，同步到浏览器失败，请及时解决冲突`);
            return;
        }
        try {
            const calsResult = (0, cals_1.codeToCals)(codeList, 'v0');
            const wsMsg = {
                event: 'code-updated',
                data: {
                    appId,
                    envId,
                    clientId,
                    calsData: calsResult,
                },
            };
            this.wedaAppClients[appId][clientId].websocket.send(JSON.stringify(wsMsg));
            if (this.wedaAppClients[appId][clientId].changedFiles.length > 0) {
                this.Logger.info(appId, clientId, '代码同步: 本地 >>> WeDa 在线编辑器');
                this.Logger.info(appId, clientId, '涉及代码文件:');
                this.Logger.info(appId, clientId, (0, lodash_1.uniqBy)(this.wedaAppClients[appId][clientId].changedFiles, 'filePath').map((item) => item.filePath).join('\n'), true);
            }
            this.wedaAppClients[appId][clientId].changedFiles = [];
            this.wedaAppClients[appId][clientId].canSync = true;
        }
        catch (e) {
            debug(e);
            this.Logger.error(appId, clientId, '【转换异常】请检查本地的 JSON 文件的内容是否为正确的 JSON 格式');
        }
    }
    async syncDTS(appId, envId, dts) {
        const projectPath = this.getProjectPath(appId, envId);
        return fs_extra_1.default.writeFile(path_1.default.resolve(projectPath, 'weda-app.d.ts'), dts, 'utf8');
    }
    listen(port = 3000) {
        this.app.listen(port);
    }
    close() {
        return Promise.all([this._closeServer(), this._closeWatches()]);
    }
    _closeServer() {
        return new Promise((resolve, reject) => {
            // 先 close 所有的连接才能正常 close server
            for (const client of this.app.ws.server.clients) {
                client.close();
            }
            this.app.server.close(function (error) {
                if (error)
                    reject(error);
                else
                    resolve('done');
            });
        });
    }
    _closeWatches() {
        return Promise.all(Object.values(this.wedaAppClients).map((appItem) => {
            return Promise.all(Object.values(appItem).map((client) => {
                return client.watcher && client.watcher.close();
            }));
        }));
    }
}
exports.KoaWSServer = KoaWSServer;
