"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.hasOwn = exports.fileToCode = exports.initLowcodeCloudApiEnv = exports.showCliVersion = exports.debug = exports.findFiles = exports.pathEscape = void 0;
const os_1 = __importDefault(require("os"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const RawDebug = __importStar(require("debug"));
const chalk_1 = __importDefault(require("chalk"));
const package_json_1 = __importDefault(require("../../package.json"));
const glob_1 = __importDefault(require("glob"));
function pathEscape(path) {
    if (os_1.default.platform() !== 'win32') {
        return path;
    }
    return path.replace(/\\/g, '\\\\');
}
exports.pathEscape = pathEscape;
function findFiles(name, dir, recursion = false, ignoreRegex = /node_modules/) {
    const paths = getFilePaths(dir, recursion, ignoreRegex);
    return paths
        .map((filePath) => path_1.default.relative(dir, filePath))
        .filter((filePath) => {
        if (name instanceof RegExp) {
            return name.test(filePath);
        }
        else {
            return name === filePath;
        }
    });
}
exports.findFiles = findFiles;
function getFilePaths(dir, recursion = false, ignoreRegex) {
    const files = fs_extra_1.default.readdirSync(dir, { withFileTypes: true });
    const paths = [];
    files.forEach((file) => {
        const resolvePath = path_1.default.resolve(dir, file.name);
        if (file.isFile()) {
            paths.push(resolvePath);
        }
        else if (file.isDirectory() && recursion) {
            if (!ignoreRegex || !ignoreRegex.test(resolvePath)) {
                paths.push(...getFilePaths(resolvePath, recursion, ignoreRegex));
            }
        }
    });
    return paths;
}
exports.debug = RawDebug.debug('weda-cli');
function showCliVersion() {
    console.log(chalk_1.default.gray(`LowCode CLI ${package_json_1.default.version}`));
}
exports.showCliVersion = showCliVersion;
const initLowcodeCloudApiEnv = (info) => {
    var _a, _b;
    if ((0, exports.hasOwn)(info, 'privateSettings') && info.privateSettings && ((_a = info.privateSettings.endpoints) === null || _a === void 0 ? void 0 : _a.cliApi)) {
        process.env.CLOUDBASE_LOWCODE_CLOUDAPI_URL = (_b = info.privateSettings.endpoints) === null || _b === void 0 ? void 0 : _b.cliApi;
    }
    if (process.env.CLOUDBASE_LOWCODE_CLOUDAPI_URL === undefined) {
        // 没设置的时候才才设置，方便覆盖
        process.env.CLOUDBASE_LOWCODE_CLOUDAPI_URL = 'https://lcap.cloud.tencent.com/api/v1/cliapi';
    }
};
exports.initLowcodeCloudApiEnv = initLowcodeCloudApiEnv;
/**
 * 指定目录下的文件转代码
 */
async function fileToCode(dirPath, onlyPath = false) {
    const result = await new Promise((resolve, reject) => {
        // 只包含 js, json, less 文件，其它忽略（因为不属于 weda 的文件）
        (0, glob_1.default)(path_1.default.resolve(dirPath, '**', '*.@(js|json|less)'), {}, async function (err, files) {
            if (err)
                reject(err);
            else {
                const codeList = await Promise.all(files.map(async (filePath) => {
                    const codePath = path_1.default.relative(dirPath, filePath);
                    const codeContent = onlyPath ? '' : await fs_extra_1.default.readFile(filePath, 'utf8');
                    return {
                        path: codePath.replace(/\\/g, '/'),
                        code: codeContent,
                    };
                }));
                resolve(codeList);
            }
        });
    });
    return result;
}
exports.fileToCode = fileToCode;
const hasOwn = (obj, name) => {
    return Object.prototype.hasOwnProperty.call(obj, name);
};
exports.hasOwn = hasOwn;
