"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCssLoader = void 0;
const mini_css_extract_plugin_1 = __importDefault(require("mini-css-extract-plugin"));
const getCSSModuleLocalIdent_1 = __importDefault(require("./getCSSModuleLocalIdent"));
const postcssPxtorem = require('postcss-pxtorem');
/**
 * 获取 CSS 文件相关 loader 配置
 *
 * 1. 正常情况下，使用 css-loader 和 ExtractCssChunks 来抽离样式文件
 * 2. 在将组件库源码构建为 npm 包时，为了充分利用 TreeShaking 功能，使用 tsc 来构建TS和JS文件
 * js文件中依赖的 css/less/sass 等文件采取只编译不 Pack 的方式，采取原位替换内容的方式输出
 * 此时会将环境变量为 LOWCODE_CLI_CSS_TRANSPILE_ONLY 设置为 true
 * @returns RuleSetUse
 */
function getCssLoader() {
    const loaders = [];
    if (!process.env.LOWCODE_CLI_CSS_TRANSPILE_ONLY || process.env.LOWCODE_CLI_CSS_TRANSPILE_ONLY === 'false') {
        loaders.push({
            loader: mini_css_extract_plugin_1.default.loader,
        });
    }
    else {
        loaders.push(...[
            {
                loader: require.resolve('raw-loader'),
            },
            {
                loader: require.resolve('./loaders/css-extract-in-situ-loader'),
            },
            {
                loader: require.resolve('extract-loader'),
            },
        ]);
    }
    loaders.push({
        loader: require.resolve('css-loader'),
        options: {
            modules: {
                auto: true,
                getLocalIdent: getCSSModuleLocalIdent_1.default,
            },
        },
    }, {
        loader: require.resolve('postcss-loader'),
        options: {
            ident: 'postcss',
            plugins: () => [
                postcssPxtorem({
                    rootValue: 14,
                    propList: ['*'],
                }),
            ],
        },
    });
    return loaders;
}
exports.getCssLoader = getCssLoader;
