"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebpackService = void 0;
/**
 * 复合组件库和基础组件库的打包流程基本一致
 * 所以, 把打包模块抽离出来共用
 */
const webpack_1 = __importDefault(require("webpack"));
const toolbox_1 = require("@cloudbase/toolbox");
const config_1 = require("./config");
class WebpackService {
    constructor(info) {
        this.info = info;
    }
    async webpack() {
        // 拼装WebpackConfig
        const config = this.createConfig();
        // 启动Webpack
        return this.runWebpack(config);
    }
    createConfig() {
        const config = (0, config_1.getWebpackConfig)(this.info);
        return config;
    }
    async runWebpack(config) {
        return new Promise((resolve, reject) => {
            const callback = (err, stats) => {
                if (err) {
                    reject(new toolbox_1.CloudBaseError(`组件库打包失败: ${err}`));
                }
                if (stats.hasErrors()) {
                    const { errors } = stats.toJson();
                    const message = errors.map((error) => error.details || error.stack || error.message).join('\n');
                    reject(new toolbox_1.CloudBaseError(`组件库打包失败: ${message}`));
                }
                resolve();
            };
            (0, webpack_1.default)(config, callback);
        });
    }
}
exports.WebpackService = WebpackService;
