"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fromEntries = require('object.fromentries');
const COMPONENTS_CONFIG_SCHEMA_VERSION = '3.0.0';
function transfer(config, mpInstance) {
    const { components, actions } = config;
    // const { components: webComponents, actions: webActions } = webInstance
    const { components: mpComponents, actions: mpActions } = mpInstance;
    // weda格式的组件配置
    const wedaComps = fromEntries(Object.entries(components).map(([name, component]) => {
        // 小程序路径, 只能在meta.platforms里指定
        let weappsPlatforms = {};
        const mpComponent = mpComponents[name];
        if (typeof mpComponent === 'string') {
            weappsPlatforms = {
                mp: {
                    tagName: name,
                    path: mpComponent,
                },
            };
        }
        else if (typeof mpComponent === 'object') {
            const { type, path } = mpComponent;
            if (type === 'native') {
                weappsPlatforms = {
                    mp: {
                        tagName: path,
                    },
                };
            }
            else if (type === 'custom') {
                weappsPlatforms = {
                    mp: {
                        tagName: name,
                        path: path,
                    },
                };
            }
        }
        const meta = {
            platforms: weappsPlatforms,
            ...component.meta,
        };
        return [
            name,
            {
                platforms: ['web', 'mp'],
                ...component,
                meta,
            },
        ];
    }));
    // weda格式的动作配置
    const wedaActions = fromEntries(Object.entries(actions).map(([name, action]) => {
        // 小程序路径, 只能在meta.platforms里指定
        let weappsPlatforms = {};
        const mpAction = mpActions[name];
        if (typeof mpAction === 'string') {
            weappsPlatforms = {
                mp: {
                    tagName: name,
                    path: mpAction,
                },
            };
        }
        const meta = {
            platforms: weappsPlatforms,
            ...action.meta,
        };
        return [
            name,
            {
                platforms: ['web', 'mp'],
                ...action,
                meta,
            },
        ];
    }));
    // weapps格式的组件配置
    const weappsComps = Object.entries(wedaComps).map(([name, component]) => {
        var _a, _b, _c, _d;
        // data to dataForm
        const dataForm = ((_a = component.data) === null || _a === void 0 ? void 0 : _a.properties) ? (_b = component.data) === null || _b === void 0 ? void 0 : _b.properties : component.data;
        // events to emitEvents
        const emitEvents = (_c = component.events) === null || _c === void 0 ? void 0 : _c.map((event) => ({
            eventName: event.name,
            name: event.title,
            detail: event.detail,
        }));
        // meta to meta
        const meta = {
            ...component.meta,
            desc: (_d = component.meta) === null || _d === void 0 ? void 0 : _d.description,
        };
        return {
            name,
            type: 'component',
            dataForm,
            emitEvents,
            methods: component.methods,
            meta,
            compConfig: component.compConfig,
            isContainer: component.isContainer,
            configMeta: component.configMeta,
            defaultStyles: component.defaultStyles,
        };
    });
    // weapps格式的动作配置
    const weappsActions = Object.entries(wedaActions).map(([name, action]) => {
        var _a, _b, _c;
        // data to dataForm
        const dataForm = ((_a = action.data) === null || _a === void 0 ? void 0 : _a.properties) ? (_b = action.data) === null || _b === void 0 ? void 0 : _b.properties : action.data;
        // meta to meta
        const meta = {
            ...action.meta,
            desc: (_c = action.meta) === null || _c === void 0 ? void 0 : _c.description,
        };
        return {
            name,
            type: 'action',
            dataForm,
            meta,
        };
    });
    return {
        schemaVersion: COMPONENTS_CONFIG_SCHEMA_VERSION,
        components: weappsComps,
        actions: weappsActions,
        plugins: [],
        lcds: {
            schemaVersion: COMPONENTS_CONFIG_SCHEMA_VERSION,
            components: wedaComps,
            actions: wedaActions,
        },
    };
}
exports.default = transfer;
